import java.util.*;
import java.io.*;

/* The unit of concentration is M. The unit of time = minute. */

class Data {
	private static double[][] data;

	public static double time(int index) {
		return data[index][0];
	}
	public static double conc(int index) {
		return data[index][1];
	}
	public static int numberOfData() {
		return data.length;
	}

	protected Data(String filename) {
		try {
			String str1;
			FileReader in1 = new FileReader(filename);
			BufferedReader b1 = new BufferedReader(in1);
			int counter1 = 0;
			while((str1 = b1.readLine()) != null)
				counter1++;
			data = new double[counter1][];
			for(int i = 0; i < counter1; i++)
				data[i] = new double[2];
			b1.close();
			in1.close();
		} catch(IOException e) {
			System.err.println("INPUT ERROR.");
		}

		try {
			String str2;
			FileReader in2 = new FileReader(filename);
			BufferedReader b2 = new BufferedReader(in2);
			int counter2 = 0;
			while((str2 = b2.readLine()) != null) {
				String[] temp = str2.split("\t");
				for(int i = 0; i < 2; i++)
					data[counter2][i] = Double.parseDouble(temp[i]);
				counter2++;
			}
			b2.close();
			in2.close();
		} catch(IOException e) {
			System.err.println("INPUT ERROR.");
		}
	}
}

class Individual {
	private double vmax;
	private double km;

	public double vmax() {
		return this.vmax;
	}
	public double km() {
		return this.km;
	}
	public void set_vmax(double vmax) {
		this.vmax = vmax;
	}
	public void set_km(double km) {
		this.km = km;
	}

/*	protected Individual(double vmax, double km) {
		this.vmax(vmax);
		this.km(km);
	}	*/
}

class DifferentialEvolution {
	private static LinkedList<Individual> population = new LinkedList<Individual>();
	private static Random rnd = new Random();
	private static final int PopulationSize = 15;
	private static final double F = 0.50;
	private static final double CR = 0.50;
	private static final int N = 2;

	private static final double InitConc = 10.0;

	private double costFunction(Individual in) {
		int minutes = 30;
		int steps = 30000;
		double delta = (double)minutes/(double)steps;
		double[] substrate = new double[steps + 1];
		substrate[0] = InitConc;
		for(int i = 0; i < steps; i++)
			substrate[i + 1] = substrate[i] - delta*in.vmax()*substrate[i]/(in.km() + substrate[i]);

		double cost = 0.0;
		for(int i = 0; i < Data.numberOfData(); i++) {
			int index = (int)(Data.time(i)*(steps/minutes));
			cost += Math.abs(substrate[index] - Data.conc(i))/Data.conc(i);
		}
		if(in.vmax() < 0.0)
			cost += 100.0*(1.0 - in.vmax());
		if(in.km() < 0.0)
			cost += 100.0*(1.0 - in.km());
		return cost;
	}

	protected DifferentialEvolution() {
		new Data("in.txt");

		rnd.setSeed(System.currentTimeMillis());
		int i = 0;
		while(i < PopulationSize) {
			Individual individual = new Individual();
			individual.set_vmax(Math.abs(1.0 + rnd.nextGaussian()));
			individual.set_km(Math.abs(1.0 + rnd.nextGaussian()));
			population.add(individual);
			i++;
		}

		int j = 0;
		for(i = 0; i < 100000; i++) {
			while(j < PopulationSize) {
				int x = rnd.nextInt(PopulationSize);
				int a, b, c;
				
				do {
					a = rnd.nextInt(PopulationSize);
				} while(a == x);
				do {
					b = rnd.nextInt(PopulationSize);
				} while(b == x || b == a);
				do {
					c = rnd.nextInt(PopulationSize);
				} while(c == x || c == a || c == b);

				int R = rnd.nextInt(N);
				Individual original = population.get(x);
				Individual candidate = new Individual();
				Individual individual1 = population.get(a);
				Individual individual2 = population.get(b);
				Individual individual3 = population.get(c);

				if(rnd.nextInt(N) == R || (10.0 + rnd.nextGaussian())%1 < CR)
					candidate.set_vmax(individual1.vmax() + F*(individual2.vmax() - individual3.vmax()));
				else
					candidate.set_vmax(original.vmax());
				if(rnd.nextInt(N) == R || (10.0 + rnd.nextGaussian())%1 < CR)
					candidate.set_km(individual1.km() + F*(individual2.km() - individual3.km()));
				else
					candidate.set_km(original.km());

				if(costFunction(original) > costFunction(candidate)) {
					population.remove(original);
					population.add(candidate);
				}
				j++;
			}
		}

		i = 0;
		Individual bestFitness = new Individual();
		bestFitness.set_vmax(1.0);
		bestFitness.set_km(1.0);
		while(i < PopulationSize) {
			Individual individual = population.get(i);
			if(costFunction(bestFitness) > costFunction(individual))
				bestFitness = individual;
			i++;
		}

		System.out.print("Vmax:");
		System.out.println(bestFitness.vmax());
		System.out.print("Km:");
		System.out.println(bestFitness.km());
	}
}

public class Uricase {
	public static void main(String[] args) {
		new DifferentialEvolution();
	}
}
